@@ -76,10 +76,9 @@ import time
 import os
 from collections import namedtuple
 from datetime import datetime
-from unittest.mock import MagicMock
 
 import easypy._multithreading_init  # noqa; make it initialize the threads tree
-from easypy.exceptions import PException
+from easypy.exceptions import PException, apply_timestamp
 from easypy.gevent import is_module_patched, non_gevent_sleep, defer_to_thread
 from easypy.humanize import IndentableTextBuffer, time_duration, compact
 from easypy.humanize import format_thread_stack, yesno_to_bool
@@ -96,8 +95,10 @@ DISABLE_CONCURRENCY = yesno_to_bool(os.getenv("EASYPY_DISABLE_CONCURRENCY", "no"
 this_module = import_module(__name__)
 THREADING_MODULE_PATHS = [threading.__file__]
 
+IS_GEVENT = False
 
 if is_module_patched("threading"):
+    IS_GEVENT = True
     import gevent
     MAX_THREAD_POOL_SIZE *= 100  # these are not threads anymore, but greenlets. so we allow a lot of them
     THREADING_MODULE_PATHS.append(gevent.__path__[0])
@@ -453,9 +454,11 @@ class Futures(list):
         """
         frames = dict(iter_thread_frames())
         for i, future in enumerate(futures, 1):
+            thread_ident = future.ctx['thread_ident']
             try:
-                frame = frames[future.ctx['thread_ident']]
+                frame = frames[thread_ident]
             except KeyError:
+                assert False, frames
                 frame = None  # this might happen in race-conditions with a new thread starting
             if not verbose or not frame:
                 if frame:
@@ -947,7 +950,7 @@ class concurrent(object):
                 self.threadname = "anon-%X" % id(self)
 
         real_thread_no_greenlet = kwargs.pop('real_thread_no_greenlet', False)
-        if is_module_patched("threading"):
+        if IS_GEVENT:
             # in case of using apply_gevent_patch function - use this option in order to defer some jobs to real threads
             self.real_thread_no_greenlet = real_thread_no_greenlet
         else:
@@ -989,13 +992,14 @@ class concurrent(object):
         except ProcessExiting as exc:
             _logger.debug(exc)
             raise
-        except (KeyboardInterrupt, Exception) as exc:
+        except KeyboardInterrupt as exc:
+            _logger.silent_exception("KeyboardInterrupt in thread running %s:", self.func)
+            self.exc = apply_timestamp(exc)
+            if IS_GEVENT:
+                raise  # in gevent we should let this exception propagate to the main greenlet
+        except Exception as exc:
             _logger.silent_exception("Exception in thread running %s: %s (traceback can be found in debug-level logs)", self.func, type(exc))
-            self.exc = exc
-            try:
-                exc.timestamp = time.time()
-            except Exception:
-                pass
+            self.exc = apply_timestamp(exc)
         finally:
             stack.close()
 
@@ -1006,11 +1010,11 @@ class concurrent(object):
     def wait(self, timeout=None):
         if self.real_thread_no_greenlet:
             # we can't '.wait' on this gevent event object, so instead we test it and sleep manually:
-            if self.stopper.is_set():
-                return True
-            non_gevent_sleep(timeout)
-            if self.stopper.is_set():
-                return True
+            timer = Timer(expiration=timeout)
+            while not timer.expired:
+                if self.stopper.is_set():
+                    return True
+                non_gevent_sleep(0.1)
             return False
         return self.stopper.wait(timeout)
 
@@ -1039,7 +1043,7 @@ class concurrent(object):
             yield self
             return
 
-        if self.real_thread_no_greenlet:
+        if self.real_thread_no_greenlet and IS_GEVENT:
             _logger.debug('sending job to a real OS thread')
             self._join = defer_to_thread(func=self._logged_func, threadname=self.threadname)
         else:
